from datetime import datetime

import pytz

from src.sundata import LightingInformation, LightPeriod, Position

zone = pytz.UTC


def test_default_sunrise_sunset():
    position = Position(51.772938, 0.102310)
    a_date = datetime(2023, 2, 13, 12, 00)
    data = LightingInformation(position, a_date)
    data.calculate()
    assert data.sunrise == datetime(2023, 2, 13, 7, 25).astimezone(zone)
    assert data.sunset == datetime(2023, 2, 13, 17, 3).astimezone(zone)


def test_default_sunrise_sunset_DAYLIGHT():
    position = Position(51.772938, 0.102310)
    a_date = datetime(2023, 2, 13, 12, 00)
    data = LightingInformation(position, a_date)
    data.calculate()
    assert data.sunrise == datetime(2023, 2, 13, 7, 25).astimezone(zone)
    assert data.sunset == datetime(2023, 2, 13, 17, 3).astimezone(zone)


def test_default_sunrise_sunset_CIVIL():
    position = Position(51.772938, 0.102310)
    a_date = datetime(2023, 2, 13, 12, 00)
    data = LightingInformation(position, a_date)
    data.calculate(LightPeriod.CIVIL)
    assert data.sunrise == datetime(2023, 2, 13, 7, 19).astimezone(zone)
    assert data.sunset == datetime(2023, 2, 13, 17, 10).astimezone(zone)


def test_default_sunrise_sunset_():
    position = Position(51.772938, 0.102310)
    a_date = datetime(2023, 2, 13, 12, 00)
    data = LightingInformation(position, a_date)
    data.calculate(LightPeriod.NAUTICAL)
    assert data.sunrise == datetime(2023, 2, 13, 6, 43).astimezone(zone)
    assert data.sunset == datetime(2023, 2, 13, 17, 45).astimezone(zone)


def test_default_sunrise_sunset_():
    position = Position(51.772938, 0.102310)
    a_date = datetime(2023, 2, 13, 12, 00)
    data = LightingInformation(position, a_date)
    data.calculate(LightPeriod.ASTRO)
    assert data.sunrise == datetime(2023, 2, 13, 6, 4).astimezone(zone)
    assert data.sunset == datetime(2023, 2, 13, 18, 25).astimezone(zone)


def test_default_sunrise_sunset_():
    position = Position(51.772938, 0.102310)
    a_date = datetime(2023, 2, 13, 12, 00)
    data = LightingInformation(position, a_date)
    data.calculate(LightPeriod.NIGHT)
    assert data.sunrise == datetime(2023, 2, 13, 5, 25).astimezone(zone)
    assert data.sunset == datetime(2023, 2, 13, 19, 4).astimezone(zone)



import pytest

from src.sundata import LightPeriod, lighting_is


@pytest.mark.parametrize("angles", [0, 1, 2, 3, 4, 10, 20, 30, 40, 50])
def test_get_day(angles):
    assert lighting_is(angles) == LightPeriod.DAY


@pytest.mark.parametrize("angles", [-0.0001, -1, -2, -3, -4, -5, -5.999999999, -6])
def test_get_civil(angles):
    assert lighting_is(angles) == LightPeriod.CIVIL


@pytest.mark.parametrize("angles", [-6.00000001, -7, -8, -9, -10, -11, -11.999999999])
def test_get_nautiucal(angles):
    assert lighting_is(angles) == LightPeriod.NAUTICAL


@pytest.mark.parametrize("angles", [-12.0000001, -13, -14, -15, -16, -17, -18])
def test_get_astro(angles):
    assert lighting_is(angles) == LightPeriod.ASTRO


@pytest.mark.parametrize(
    "angles", [-18.0000001, -19, -20, -21, -22, -23, -24, -25, -26, -27, -28, -29, -30]
)
def test_get_night(angles):
    assert lighting_is(angles) == LightPeriod.NIGHT



from datetime import datetime

from src.sundata import (
    LightPeriod,
    Position,
    get_lighting_period_after,
    get_lighting_period_before,
)


def test_get_night_before_date():
    pos = Position(51.772938, 0.102310)
    sunrise = datetime(2023, 2, 12, 8, 3)
    night_end = get_lighting_period_before(pos, sunrise, LightPeriod.NIGHT)
    assert night_end == datetime(2023, 2, 12, 5, 26)


def test_get_nght_after_date():
    pos = Position(51.772938, 0.102310)
    sunset = datetime(2023, 2, 12, 17, 7)
    night_start = get_lighting_period_after(pos, sunset, LightPeriod.NIGHT)
    assert night_start == datetime(2023, 2, 12, 19, 2)



from datetime import datetime
from src.sundata import Position, get_sun_altitude

def test_get_altitude():
    pos = Position(51.772938, 0.102310)
    when = datetime(2023, 2, 12, 23, 3)
    assert get_sun_altitude(pos, when) == -49.20252290488592


def test_get_altitude_zero():
    pos = Position(51.772938, 0.102310)
    when = datetime(2023, 2, 12, 17, 3)
    sun = get_sun_altitude(pos, when)
    assert int(sun) == 0


def test_get_altitude_20():
    pos = Position(51.772938, 0.102310)
    when = datetime(2023, 2, 12, 19, 16)
    sun = get_sun_altitude(pos, when)
    assert int(sun) == -20



from dataclasses import dataclass
from datetime import datetime, timedelta
from enum import Enum

import astropy.coordinates as coord
import astropy.units as u
import pytz
from astropy.time import Time
from suntime import Sun


@dataclass
class Position:
    latitude: float
    longitude: float


class LightPeriod(Enum):
    NIGHT = 0
    ASTRO = 1
    NAUTICAL = 2
    CIVIL = 3
    DAY = 4


class LightingInformation:
    sunrise: datetime
    sunset: datetime
    location: Position
    set_date: datetime
    utc = pytz.UTC

    def __init__(self, position: Position, a_datetime: datetime) -> None:
        self.location = position
        self.set_date = a_datetime.astimezone(self.utc)

    def calculate(self, lighting_period: LightPeriod = LightPeriod.DAY):
        sun = Sun(self.location.latitude, self.location.longitude)
        self.sunrise = sun.get_local_sunrise_time(self.set_date).astimezone(self.utc)
        sunrise_angle = get_sun_altitude(self.location, self.sunrise)
        if sunrise_angle < 0 and lighting_period == LightPeriod.DAY:
            self.sunrise = get_lighting_period_after(
                self.location, self.sunrise, lighting_period
            )
        else:
            self.sunrise = get_lighting_period_before(
                self.location, self.sunrise, lighting_period
            )

        self.sunset = sun.get_local_sunset_time(self.set_date).astimezone(self.utc)
        sunset_angle = get_sun_altitude(self.location, self.sunset)

        if sunset_angle < 0 and lighting_period == LightPeriod.DAY:
            self.sunset = get_lighting_period_before(
                self.location, self.sunset, lighting_period
            )
        else:
            self.sunset = get_lighting_period_after(
                self.location, self.sunset, lighting_period
            )


def get_sun_altitude(position: Position, when: datetime) -> float:
    earth_location = coord.EarthLocation(
        lon=position.longitude * u.deg, lat=position.latitude * u.deg
    )
    when = Time(when, format="datetime", scale="utc")
    altazframe = coord.AltAz(obstime=when, location=earth_location)
    sunaltaz = coord.get_sun(when).transform_to(altazframe)
    return sunaltaz.alt.max().value


def lighting_is(sun_altitude: float) -> LightPeriod:
    if sun_altitude >= 0:
        return LightPeriod.DAY
    if sun_altitude < -18:
        return LightPeriod.NIGHT
    if sun_altitude < 0 and sun_altitude >= -6:
        return LightPeriod.CIVIL
    if sun_altitude < -6 and sun_altitude >= -12:
        return LightPeriod.NAUTICAL
    if sun_altitude < -12 and sun_altitude >= -18:
        return LightPeriod.ASTRO


def get_lighting_period_after(
    position: Position, when: datetime, period: LightPeriod
) -> datetime:
    lighting = when

    while period.value != lighting_is(get_sun_altitude(position, lighting)).value:
        lighting = lighting + timedelta(minutes=1)

    return lighting


def get_lighting_period_before(
    position: Position, when: datetime, period: LightPeriod
) -> datetime:
    lighting = when

    while period.value != lighting_is(get_sun_altitude(position, lighting)).value:
        lighting = lighting - timedelta(minutes=1)

    return lighting
